/**This computes the energy of a set of points in CP2
   with respect to an elliptic curve. The energy 
   functional is designed to exact a high penalty
   to points not near the elliptic curve*/


public class Energy {



    /**This is a cheap update routine for points in the plane.
       The number d controls the gentleness of the operation.*/

    public static void update(double d,Complex[] z,int k,int L) {
	int i=nearestPointIndex(z,k,L);
	Complex w=bestRepresentative(z[k],z[i]);
	Complex z1=new Complex();
	z1.x=(1+d)*z[k].x-d*w.x;
	z1.y=(1+d)*z[k].y-d*w.y;
	Complex middle=new Complex(.5,.5);
	z[k]=bestRepresentative(middle,z1);
    }

    /**gets the nearest point index*/

    public static int nearestPointIndex(Complex[] z,int k,int L) {
	double min=10;
	int index=-1;
	for(int i=0;i<L;++i) {
	    if(i!=k) {
		Complex w=bestRepresentative(z[k],z[i]);
		double test=Complex.dist(z[k],w);
		if(test<min) {
		    min=test;
		    index =i;
		}
	    }
	}
	return index;
    }

	/**Gets the best representative of z2 in the universal
           cover of the square torus w.r.t. z1*/

	public static Complex bestRepresentative(Complex z1,Complex z2) {
	    double min=10;
	    Complex W2=new Complex(z2);
	    for(int i=-2;i<3;++i) {
		for(int j=-2;j<3;++j) {
		    Complex w2=new Complex(z2.x+i,z2.y+j);
		    double test=Complex.dist(z1,w2);
		    if(min>test) {
                         min=test;
			 W2=new Complex(w2);
		    }
		}
	    }
	    return W2;
	}


}


